@@ -1,7 +1,6 @@
-import asyncio
 import linecache
 
-from .state import Initialized
+from .state import Machine
 
 
 # __________________________________________________________________||
@@ -24,11 +23,8 @@ class Nextline:
     """
 
     def __init__(self, statement):
-        self._condition_finish = asyncio.Condition()
-        self._condition_close = asyncio.Condition()
-
-        self._state = Initialized(statement)
-        self.registry = self._state.registry
+        self.machine = Machine(statement)
+        self.registry = self.machine.registry
 
     def __repr__(self):
         # e.g., "<Nextline 'running'>"
@@ -37,42 +33,36 @@ class Nextline:
     @property
     def global_state(self) -> str:
         """state, e.g., "initialized", "running", "exited", "finished", "closed" """
-        return self._state.name
+        return self.machine.state_name
 
     def run(self):
-        """run the script"""
-        self._state = self._state.run()
-        self._task_exited = asyncio.create_task(self._exited())
-
-    async def _exited(self):
-        """receive the exited state, to be scheduled in run()."""
-        self._state = await self._state.exited()
+        """execute the script"""
+        self.machine.run()
 
     async def finish(self):
-        """finish the script execution
+        """wait until the script execution exits
 
         wait for the script execution in another thread to exit and
         join the thread.
 
         """
-        await self._task_exited
-        async with self._condition_finish:
-            self._state = await self._state.finish()
+        await self.machine.finish()
 
     def exception(self):
-        return self._state.exception()
+        """uncaught exeption from the last run"""
+        return self.machine.exception()
 
     def result(self):
-        self._state.result()
+        """return value of the last run. always None"""
+        self.machine.result()
 
     def reset(self, statement=None):
-        """reset the state"""
-        self._state = self._state.reset(statement=statement)
+        """prepare for run"""
+        self.machine.reset(statement=statement)
 
     async def close(self):
-        """close the nextline"""
-        async with self._condition_close:
-            self._state = await self._state.close()
+        """end gracefully"""
+        await self.machine.close()
 
     async def subscribe_global_state(self):
         # wish to be able to write with "yield from" but not possible
@@ -92,7 +82,7 @@ class Nextline:
             yield y
 
     def send_pdb_command(self, thread_asynctask_id, command):
-        self._state.send_pdb_command(thread_asynctask_id, command)
+        self.machine.send_pdb_command(thread_asynctask_id, command)
 
     def get_source(self, file_name=None):
         if not file_name or file_name == self.registry.get("script_file_name"):
