@@ -9,6 +9,97 @@ from .exec_ import exec_with_trace
 SCRIPT_FILE_NAME = "<string>"
 
 
+# __________________________________________________________________||
+class Machine:
+    """State machine
+
+                 .-------------.
+            .--->| Initialized |---.
+    reset() |    '-------------'   |
+            |      |   | run()     |
+            |------'   |           |
+            |          v           |
+            |    .-------------.   |
+            |    |   Running   |   |
+            |    '-------------'   |
+            |          | _exited() |
+            |          |           |
+            |          v           |
+            |    .-------------.   |
+            |    |   Exited    |   |
+            |    '-------------'   |
+            |          | finish()  |
+            |          |           |
+            |          V           |
+            |    .-------------.   |
+            |----|   Finished  |   |
+            |    '-------------'   |
+            |          |-----------'
+            |          | close()
+            |          V
+            |    .-------------.
+            '----|   Closed    |
+                 '-------------'
+
+    """
+
+    def __init__(self, statement):
+        self.condition_finish = asyncio.Condition()
+        self.condition_close = asyncio.Condition()
+
+        self.state = Initialized(statement)
+        self.registry = self.state.registry
+
+    def __repr__(self):
+        # e.g., "<Machine 'running'>"
+        return f"<{self.__class__.__name__} {self.state_name!r}>"
+
+    @property
+    def state_name(self) -> str:
+        """e.g., "initialized", "running","""
+        return self.state.name
+
+    def run(self):
+        """Enter the running state"""
+        self.state = self.state.run()
+        self._task_exited = asyncio.create_task(self._exited())
+
+    async def _exited(self):
+        """Enter the exited state
+
+        This method is scheduled as a task in run().
+
+        Bofore changing the state to "exited", this method waits for the script
+        execution (in another thread) to exit.
+        """
+
+        self.state = await self.state.exited()
+
+    def send_pdb_command(self, thread_asynctask_id, command):
+        self.state.send_pdb_command(thread_asynctask_id, command)
+
+    async def finish(self):
+        """Enter the finished state"""
+        await self._task_exited
+        async with self.condition_finish:
+            self.state = await self.state.finish()
+
+    def exception(self):
+        return self.state.exception()
+
+    def result(self):
+        self.state.result()
+
+    def reset(self, statement=None):
+        """Enter the initialized state"""
+        self.state = self.state.reset(statement=statement)
+
+    async def close(self):
+        """Enter the closed state"""
+        async with self.condition_close:
+            self.state = await self.state.close()
+
+
 # __________________________________________________________________||
 class StateObsoleteError(Exception):
     """Operation on an obsolete state object."""
